summaryrefslogtreecommitdiff
path: root/job.c
diff options
context:
space:
mode:
authorLennart Poettering <lennart@poettering.net>2010-01-20 02:12:51 +0100
committerLennart Poettering <lennart@poettering.net>2010-01-20 02:12:51 +0100
commite5b5ae50f02d7814f37b15329da2adce0fb30825 (patch)
treeacb76787259f194fb228c9401d5544cb97aa3937 /job.c
parentf00b3eda2f92b2dcd13c50749ec5e7fce6720b52 (diff)
implement transaction engine
Diffstat (limited to 'job.c')
-rw-r--r--job.c105
1 files changed, 100 insertions, 5 deletions
diff --git a/job.c b/job.c
index 2f7912d262..3e8cca1e8f 100644
--- a/job.c
+++ b/job.c
@@ -20,7 +20,7 @@ Job* job_new(Manager *m, JobType type, Name *name) {
j->type = type;
j->name = name;
/* We don't link it here, that's what job_link() is for */
return j;
}
@@ -37,13 +37,96 @@ void job_free(Job *j) {
hashmap_remove(j->manager->jobs, UINT32_TO_PTR(j->id));
}
- hashmap_remove(j->manager->jobs_to_add, j->name);
- set_remove(j->manager->jobs_to_remove, j);
+ manager_transaction_delete_job(j->manager, j);
- /* Free data and next 'smaller' objects */
free(j);
}
+JobDependency* job_dependency_new(Job *subject, Job *object, bool matters) {
+ JobDependency *l;
+
+ assert(object);
+
+ /* Adds a new job link, which encodes that the 'subject' job
+ * needs the 'object' job in some way. If 'subject' is NULL
+ * this means the 'anchor' job (i.e. the one the user
+ * explcitily asked for) is the requester. */
+
+ if (!(l = new(JobDependency, 1)))
+ return NULL;
+
+ l->subject = subject;
+ l->object = object;
+ l->matters = matters;
+
+ if (subject) {
+ l->subject_next = subject->subject_list;
+ subject->subject_list = l;
+ } else {
+ l->subject_next = object->manager->transaction_anchor;
+ object->manager->transaction_anchor = l;
+ }
+
+ if (l->subject_next)
+ l->subject_next->subject_prev = l;
+ l->subject_prev = NULL;
+
+ if ((l->object_next = object->object_list))
+ l->object_next->object_prev = l;
+ l->object_prev = NULL;
+ object->object_list = l;
+
+ return l;
+}
+
+void job_dependency_free(JobDependency *l) {
+ assert(l);
+
+ if (l->subject_prev)
+ l->subject_prev->subject_next = l->subject_next;
+ else if (l->subject)
+ l->subject->subject_list = l->subject_next;
+ else
+ l->object->manager->transaction_anchor = l->subject_next;
+
+ if (l->subject_next)
+ l->subject_next->subject_prev = l->subject_prev;
+
+ if (l->object_prev)
+ l->object_prev->object_next = l->object_next;
+ else
+ l->object->object_list = l->object_next;
+
+ if (l->object_next)
+ l->object_next->object_prev = l->object_prev;
+
+ free(l);
+}
+
+void job_dependency_delete(Job *subject, Job *object, bool *matters) {
+ JobDependency *l;
+
+ assert(object);
+
+ for (l = object->object_list; l; l = l->object_next) {
+ assert(l->object == object);
+
+ if (l->subject == subject)
+ break;
+ }
+
+ if (!l) {
+ if (matters)
+ *matters = false;
+ return;
+ }
+
+ if (matters)
+ *matters = l->matters;
+
+ job_dependency_free(l);
+}
+
void job_dump(Job *j, FILE*f) {
static const char* const job_type_table[_JOB_TYPE_MAX] = {
@@ -51,9 +134,9 @@ void job_dump(Job *j, FILE*f) {
[JOB_STOP] = "stop",
[JOB_VERIFY_STARTED] = "verify-started",
[JOB_RELOAD] = "reload",
+ [JOB_RELOAD_OR_START] = "reload-or-start",
[JOB_RESTART] = "restart",
[JOB_TRY_RESTART] = "try-restart",
- [JOB_RESTART_FINISH] = "restart-finish"
};
static const char* const job_state_table[_JOB_STATE_MAX] = {
@@ -71,3 +154,15 @@ void job_dump(Job *j, FILE*f) {
job_type_table[j->type],
job_state_table[j->state]);
}
+
+bool job_is_anchor(Job *j) {
+ JobDependency *l;
+
+ assert(j);
+
+ for (l = j->object_list; l; l = l->object_next)
+ if (!l->subject)
+ return true;
+
+ return false;
+}
@@ -20,7 +20,7 @@ Job* job_new(Manager *m, JobType type, Name *name) {
j->type = type;
j->name = name;
- /* We don't link it here, that's what job_link() is for */
+ /* We don't link it here, that's what job_dependency() is for */
return j;
}